import json
import os
import pkg_resources

from .schema import Schema


class Verdict(Schema):
    def __init__(self):
        self.malware_family = None
        self.scanner = None
        self.domains = None
        self.ip_addresses = None
        self.stix = None
        self.extra = None

    def add_domain(self, domain):
        if self.domains is None:
            self.domains = []

        self.domains.append(domain)
        return self

    def add_domains(self, domains):
        if self.domains is None:
            self.domains = []

        self.domains.extend(domains)
        return self

    def add_extra(self, key, value):
        if self.extra is None:
            self.extra = []

        self.extra.append((key, value))

        return self

    def add_extras(self, extras):
        if self.extra is None:
            self.extra = []

        for k, v in extras:
            self.extra.append((k, v))

        return self

    def add_ip_address(self, ip_address):
        if self.ip_addresses is None:
            self.ip_addresses = []

        self.ip_addresses.append(ip_address)
        return self

    def add_ip_addresses(self, ip_addresses):
        if self.ip_addresses is None:
            self.ip_addresses = []

        self.ip_addresses.extend(ip_addresses)
        return self

    def add_stix_signature(self, schema, signature):
        if self.stix is None:
            self.stix = []

        self.stix.append({"schema": schema, "signature": signature})
        return self

    def add_stix_signatures(self, signatures):
        if self.stix is None:
            self.stix = []

        for schema, signature in signatures:
            self.stix.append({"schema": schema, "signature": signature})
        return self

    @classmethod
    def get_schema(cls):
        """
        Get the path to the backing schema this Metadata object users
        :return: Tuple[string, string] where first string is the path,
        and the second is the schema name
        """
        return pkg_resources.resource_filename(__name__, os.path.join('verdict.json')), 'verdict'

    def json(self):
        """
        Convert metadata implementation into json string
        :return: JSON string representing the internal type of this object
        """
        output = VerdictEncoder().encode(self)
        if not Verdict.validate(json.loads(output)):
            raise ValueError('Invalid Artifact setup')

        return output

    def set_malware_family(self, malware_family):
        self.malware_family = malware_family
        return self

    def set_scanner(self, operating_system=None, architecture=None, version=None, polyswarmclient_version=None,
                    signatures_version=None, vendor_version=None):
        scanner = {}

        if operating_system is not None or architecture is not None:
            scanner["environment"] = {
                "operating_system": operating_system,
                "architecture": architecture
            }

        if polyswarmclient_version is not None:
            scanner["polyswarmclient_version"] = polyswarmclient_version

        if version is not None:
            scanner["version"] = version

        if signatures_version is not None:
            scanner["signatures_version"] = signatures_version

        if vendor_version is not None:
            scanner['vendor_version'] = vendor_version

        self.scanner = scanner if scanner else None
        return self


class VerdictEncoder(json.JSONEncoder):
    def encode(self, obj):
        if isinstance(obj, Verdict):
            output = {
                "malware_family": obj.malware_family,
            }

            if obj.scanner:
                output["scanner"] = obj.scanner

            if obj.domains:
                output['domains'] = obj.domains

            if obj.ip_addresses:
                output['ip_addresses'] = obj.ip_addresses

            if obj.stix:
                output['stix'] = obj.stix

            if obj.extra:
                for key, value in obj.extra:
                    output[key] = value

            return json.dumps(output)



import json
import os

import pkg_resources
from jsonschema import RefResolver

from .schema import Schema


class Bounty(Schema):
    def __init__(self, ):
        self.artifacts = []

    def add_file_artifact(self, mimetype, filename=None, filesize=None, sha256=None, sha1=None, md5=None):
        if mimetype is None:
            raise ValueError("Mimetype cannot be None")
        artifact = {
            "mimetype": mimetype,
        }
        if filename is not None:
            artifact['filename'] = filename

        if filesize is not None:
            artifact['filesize'] = filesize

        if sha256 is not None:
            artifact['sha256'] = sha256

        if sha1 is not None:
            artifact['sha1'] = sha1

        if md5 is not None:
            artifact['md5'] = md5

        self.artifacts.append(artifact)
        return self

    def add_url_artifact(self, *_args, **kwargs):
        self.artifacts.append({**kwargs})
        return self

    @classmethod
    def get_schema(cls):
        """
        Get the path to the backing schema this Metadata object users
        :return: Tuple[string, string] where first string is the path,
        and the second is the schema name
        """
        return (
            pkg_resources.resource_filename(__name__, os.path.join('bounty.json')),
            'bounty'
        )

    def json(self):
        """
        Convert metadata implementation into json string
        :return: JSON string representing the internal type of this object
        """
        output = BountyEncoder().encode(self)
        if not Bounty.validate(json.loads(output)):
            raise ValueError('Invalid Bounty setup')

        return output

    @classmethod
    def validate(cls, value, resolver=None, silent=False):
        schema_path, schema_name = cls.get_schema()
        if not os.path.exists(schema_path):
            raise FileNotFoundError('Cannot find schema_path: {schema_path}'.format(schema_path=schema_path))

        with open(schema_path) as f:
            schema = json.loads(f.read())

        resolver = RefResolver.from_schema(schema)
        return super().validate(value, resolver, silent)


class BountyEncoder(json.JSONEncoder):
    def encode(self, obj):
        if isinstance(obj, Bounty):
            return json.dumps([artifact for artifact in obj.artifacts])



import json
import logging
import os

from abc import ABC, abstractmethod
from jsonschema import validate, RefResolver, ValidationError

logger = logging.getLogger(__name__)


class Schema(ABC):
    @classmethod
    @abstractmethod
    def get_schema(cls):
        """
        Get the path to the backing schema this Metadata object users
        :return: Tuple[string, string] where first string is the path,
        and the second is the schema name
        """
        raise NotImplementedError('get_schema is not implemented')

    @abstractmethod
    def json(self):
        """
        Convert metadata implementation into json string
        :return: JSON string representing the internal type of this object
        """
        raise NotImplementedError('json is not implemented')

    @classmethod
    def validate(cls, value, resolver=None, silent=False):
        """
        Validates a JSON value against the schema
        :param value: Dict or array to compare
        :param resolver: Optional resolver for finding references
        :param silent: Silence exception if silent is True
        :return: Boolean, True if JSON matches the schema
        """
        schema_path, schema_name = cls.get_schema()
        if not os.path.exists(schema_path):
            raise FileNotFoundError('Cannot find schema_path: {schema_path}'.format(schema_path=schema_path))

        with open(schema_path) as f:
            schema = json.loads(f.read())

        if resolver is None:
            resolver = RefResolver('file://{schema_path}'.format(schema_path=schema_path), schema_name)

        try:
            validate(instance=value, schema=schema, resolver=resolver)
        except ValidationError:
            if not silent:
                logger.exception('Failed to validate schema')
            return False

        return True



import json
import os
import pkg_resources

from .schema import Schema
from .verdict import VerdictEncoder


class Assertion(Schema):
    def __init__(self, ):
        self.artifacts = []

    def add_artifact(self, artifact):
        self.artifacts.append(artifact)
        return self

    def add_artifacts(self, artifacts):
        self.artifacts.extend(artifacts)
        return self

    @classmethod
    def get_schema(cls):
        """
        Get the path to the backing schema this Metadata object users
        :return: Tuple[string, string] where first string is the path,
        and the second is the schema name
        """
        return (
            pkg_resources.resource_filename(__name__, os.path.join('assertion.json')),
            'assertion'
        )

    def json(self):
        """
        Convert metadata implementation into json string
        :return: JSON string representing the internal type of this object
        """
        if any([artifact is None for artifact in self.artifacts]):
            raise ValueError('Artifacts cannot be None')
        output = AssertionEncoder().encode(self)
        if not Assertion.validate(json.loads(output)):
            raise ValueError('Invalid Bounty setup')

        return output


class AssertionEncoder(json.JSONEncoder):
    def encode(self, obj):
        if isinstance(obj, Assertion):
            return json.dumps([json.loads(VerdictEncoder().encode(artifact)) for artifact in obj.artifacts])



from setuptools import setup, find_packages


# The README.md will be used as the content for the PyPi package details page on the Python Package Index.
with open('README.md', 'r') as readme:
    long_description = readme.read()


setup(
    name='polyswarm-artifact',
    version='1.3.1',
    description='Library containing artifact type enums and functions',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='PolySwarm Developers',
    author_email='info@polyswarm.io',
    url='https://github.com/polyswarm/polyswarm-artifact',
    license='MIT',
    python_requires='>=3.5.2,<4',
    install_requires=[
        'jsonschema'
    ],
    include_package_data=True,
    packages=find_packages('src'),
    package_dir={'': 'src'},
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: PyPy',
    ]
)
